@@ -1,5 +1,4 @@
 import torch
-import math
 from torch import nn, einsum
 from inspect import isfunction
 from functools import partial
@@ -14,6 +13,7 @@ from einops.layers.torch import Rearrange
 
 from alphafold2_pytorch.utils import *
 import alphafold2_pytorch.constants as constants
+from alphafold2_pytorch.mlm import MLM
 
 # structure module
 
@@ -358,10 +358,10 @@ class PairwiseAttentionBlock(nn.Module):
         super().__init__()
         self.outer_mean = OuterMean(dim)
 
-        self.triangle_attention_ingoing = AxialAttention(dim = dim, heads = heads, dim_head = dim_head, row_attn = True, col_attn = False, accept_edges = True)
-        self.triangle_attention_outgoing = AxialAttention(dim = dim, heads = heads, dim_head = dim_head, row_attn = False, col_attn = True, accept_edges = True, global_query_attn = global_column_attn)
-        self.triangle_multiply_ingoing = TriangleMultiplicativeModule(dim = dim, mix = 'ingoing')
+        self.triangle_attention_outgoing = AxialAttention(dim = dim, heads = heads, dim_head = dim_head, row_attn = True, col_attn = False, accept_edges = True)
+        self.triangle_attention_ingoing = AxialAttention(dim = dim, heads = heads, dim_head = dim_head, row_attn = False, col_attn = True, accept_edges = True, global_query_attn = global_column_attn)
         self.triangle_multiply_outgoing = TriangleMultiplicativeModule(dim = dim, mix = 'outgoing')
+        self.triangle_multiply_ingoing = TriangleMultiplicativeModule(dim = dim, mix = 'ingoing')
 
     def forward(
         self,
@@ -372,10 +372,10 @@ class PairwiseAttentionBlock(nn.Module):
         if exists(msa_repr):
             x = x + self.outer_mean(msa_repr)
 
-        x = self.triangle_attention_ingoing(x, edges = x, mask = mask) + x
-        x = self.triangle_attention_outgoing(x, edges = x, mask = mask) + x
-        x = self.triangle_multiply_ingoing(x, mask = mask) + x
         x = self.triangle_multiply_outgoing(x, mask = mask) + x
+        x = self.triangle_multiply_ingoing(x, mask = mask) + x
+        x = self.triangle_attention_outgoing(x, edges = x, mask = mask) + x
+        x = self.triangle_attention_ingoing(x, edges = x, mask = mask) + x
         return x
 
 class MsaAttentionBlock(nn.Module):
@@ -397,8 +397,8 @@ class MsaAttentionBlock(nn.Module):
         mask = None,
         pairwise_repr = None
     ):
-        x = self.row_attn(x, mask = mask) + x
-        x = self.col_attn(x, mask = mask, edges = pairwise_repr) + x
+        x = self.row_attn(x, mask = mask, edges = pairwise_repr) + x
+        x = self.col_attn(x, mask = mask) + x
         return x
 
 # main evoformer class
@@ -437,100 +437,16 @@ class Evoformer(nn.Module):
         for attn, ff, msa_attn, msa_ff in self.layers:
             # msa attention and transition
 
-            m = msa_attn(m, mask = msa_mask, pairwise_repr = x) + m
+            m = msa_attn(m, mask = msa_mask, pairwise_repr = x)
             m = msa_ff(m) + m
 
             # pairwise attention and transition
 
-            x = attn(x, mask = mask) + x
+            x = attn(x, mask = mask)
             x = ff(x) + x
 
         return x, m
 
-# MSA MLM
-
-def get_mask_subset_with_prob(mask, prob):
-    batch, seq_len, device = *mask.shape, mask.device
-    max_masked = math.ceil(prob * seq_len)
-
-    num_tokens = mask.sum(dim=-1, keepdim=True)
-    mask_excess = (mask.cumsum(dim=-1) > (num_tokens * prob).ceil())
-    mask_excess = mask_excess[:, :max_masked]
-
-    rand = torch.rand((batch, seq_len), device=device).masked_fill(~mask, -1e9)
-    _, sampled_indices = rand.topk(max_masked, dim=-1)
-    sampled_indices = (sampled_indices + 1).masked_fill_(mask_excess, 0)
-
-    new_mask = torch.zeros((batch, seq_len + 1), device=device)
-    new_mask.scatter_(-1, sampled_indices, 1)
-    return new_mask[:, 1:].bool()
-
-class MLM(nn.Module):
-    def __init__(
-        self,
-        dim,
-        mask_id,
-        mask_prob = 0.15,
-        random_replace_token_prob = 0.1,
-        keep_token_same_prob = 0.1,
-        exclude_token_ids = (0,)
-    ):
-        super().__init__()
-        self.to_logits = nn.Linear(dim, constants.NUM_AMINO_ACIDS)
-        self.mask_id = mask_id
-
-        self.mask_prob = mask_prob
-        self.exclude_token_ids = exclude_token_ids
-        self.keep_token_same_prob = keep_token_same_prob
-        self.random_replace_token_prob = random_replace_token_prob
-
-    def noise(self, seq, mask):
-        num_msa = seq.shape[1]
-        seq = rearrange(seq, 'b n ... -> (b n) ...')
-        mask = rearrange(mask, 'b n ... -> (b n) ...')
-
-        # prepare masks for noising sequence
-
-        excluded_tokens_mask = mask
-
-        for token_id in self.exclude_token_ids:
-            excluded_tokens_mask = excluded_tokens_mask & (seq != token_id)
-
-        mlm_mask = get_mask_subset_with_prob(excluded_tokens_mask, self.mask_prob)
-
-        # keep some tokens the same
-
-        replace_token_with_mask = get_mask_subset_with_prob(mlm_mask, 1. - self.keep_token_same_prob)
-
-        # replace with mask
-
-        seq = seq.masked_fill(mlm_mask, self.mask_id)
-
-        # generate random tokens
-
-        random_replace_token_prob_mask = get_mask_subset_with_prob(mlm_mask, (1 - self.keep_token_same_prob) * self.random_replace_token_prob)
-
-        random_tokens = torch.randint(1, constants.NUM_AMINO_ACIDS, seq.shape)
-
-        for token_id in self.exclude_token_ids:
-            random_replace_token_prob_mask = random_replace_token_prob_mask & (random_tokens != token_id)  # make sure you never substitute a token with an excluded token type (pad, start, end)
-
-        # noise sequence
-
-        noised_seq = torch.where(random_replace_token_prob_mask, random_tokens, seq)
-        noised_seq = rearrange(noised_seq, '(b n) ... -> b n ...', n = num_msa)
-        mlm_mask = rearrange(mlm_mask, '(b n) ... -> b n ...', n = num_msa)
-
-        return noised_seq, mlm_mask
-
-    def forward(self, seq_embed, original_seq, mask):
-        logits = self.to_logits(seq_embed)
-        seq_logits = logits[mask]
-        seq_labels = original_seq[mask]
-
-        loss = F.cross_entropy(seq_logits, seq_labels, reduction = 'mean')
-        return loss
-
 class Alphafold2(nn.Module):
     def __init__(
         self,
@@ -656,6 +572,7 @@ class Alphafold2(nn.Module):
 
         self.mlm = MLM(
             dim = dim,
+            num_tokens = num_tokens,
             mask_id = num_tokens, # last token of embedding is used for masking
             mask_prob = mlm_mask_prob,
             keep_token_same_prob = mlm_keep_token_same_prob,
